/*
**	SN.C
**	Version 1.00
**
**	Changes serial number of disks formatted under DOS version 4 or greater.
**
**	usage:
**		serial <drive> <serial-number>
**
**	OS:			MS-DOS 5.00
**	Compiler:	Quick C 2.5
**	Model:		small
**
**	SN is copyright 1991 by Steve Grant.  All rights reserved.  Neither the
**	source code nor the object code of SN carries any warranty, either
**	express or implied, of merchantability or of fitness for a particular
**	purpose.
**
**	Comments, questions, and suggestions may be directed to the author at:
**
**		BIX: sjgrant
**
**	Steve Grant
**	Jersey City, NJ
**	September 5, 1991
*/

#pragma		pack(1)

#include	<ctype.h>
#include	<dos.h>
#include	<errno.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#define		RESERVED_SIZE	39

struct	BOOT_RECORD_T {
	unsigned char	reserved_1[RESERVED_SIZE];
	unsigned long	serial_number;
	unsigned char	reserved_2[BUFSIZ - RESERVED_SIZE - sizeof(long)];
};

extern unsigned		read_boot_sector(unsigned char, struct BOOT_RECORD_T *);
extern unsigned		write_boot_sector(unsigned char, struct BOOT_RECORD_T *);

void	_setenvp(void);

void	main(int argc, char *argv[]) {

	struct BOOT_RECORD_T	boot_record;
	struct diskfree_t		diskfree;
	int						i;
	unsigned long			ul;

	if (_osmajor < 2) {
		fprintf(stderr, "\
SN requires DOS 2.0 or later\n\
Your DOS version is %u.%02u\n",
		_osmajor, _osminor);
	}
	if (argc != 3
		|| strlen(argv[1]) != 2
		|| !isalpha(i = argv[1][0])
		|| argv[1][1] != ':') {
		errno = EINVAL;
		fputs("usage:  sn <drive>: <serial number>\n", stderr);
		exit(errno);
	}
	i = toupper(i);
	if (_dos_getdiskfree(i - 'A' + 1, &diskfree)) {
		fprintf(stderr, "Error reading drive %c: information\n", i);
		exit(errno);
	}
	if (read_boot_sector((unsigned char) (i - 'A'), &boot_record)) {
		errno = EINVAL;
		fprintf(stderr, "Error reading drive %c: boot sector\n", i);
		exit(errno);
	}
	sscanf(argv[2], "%lX", &ul);
	boot_record.serial_number = ul;
	if (write_boot_sector((unsigned char) (i - 'A'), &boot_record)) {
		errno = EINVAL;
		fprintf(stderr, "Error writing drive %c: boot sector\n", i);
		exit(errno);
	}
	printf("Serial number %08lX successfully written to drive %c:\n", ul, i);
	exit(EZERO);
}

void	_setenvp(void) {
}
